--[[
    Subtitles — adds subtitles to Minetest.

    This module implements the Agent class, which represents a player.

    Copyright © 2022, Silver Sandstone <@SilverSandstone@craftodon.social>

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
]]


local S = subtitles.S;


subtitles.Agent = subtitles.Object:extend();

function subtitles.Agent:new(username)
    self.username = username;

    self.display = nil;
end;

function subtitles.Agent:get_player()
    return minetest.get_player_by_name(self.username);
end;

function subtitles.Agent:get_display()
    if not self.display then
        self.display = self:create_display();
    end;
    return self.display;
end;

function subtitles.Agent:create_display()
    local display_name = self:get_display_name();
    local display_type = subtitles.SubtitleDisplay.get_by_name(display_name) or subtitles.CornerSubtitleDisplay;
    return display_type(self.username);
end;

function subtitles.Agent:get_enabled()
    local player = self:get_player();
    if not player then
        return nil;
    end;
    local meta = player:get_meta();
    return meta:get('subtitles:enabled') == 'true';
end;

function subtitles.Agent:set_enabled(enabled)
    local player = self:get_player();
    assert(player);

    local meta = player:get_meta();
    meta:set_string('subtitles:enabled', enabled and 'true' or 'false');

    if not enabled then
        if self.display then
            self.display:destroy();
            self.display = nil;
        end;
    end;
end;

function subtitles.Agent:toggle_enabled()
    self:set_enabled(not self:get_enabled());
end;

function subtitles.Agent:get_display_name()
    local player = self:get_player();
    if not player then
        return subtitles.DEFAULT_DISPLAY_NAME;
    end;
    local meta = player:get_meta();
    return meta:get('subtitles:display') or subtitles.DEFAULT_DISPLAY_NAME;
end;

function subtitles.Agent:set_display_name(name)
    assert(subtitles.SubtitleDisplay.get_by_name(name), ('Invalid display mode: ‘%s’.'):format(name));
    if self.display and self.display.NAME ~= name then
        self.display:destroy();
        self.display = nil;
    end;

    local player = self:get_player();
    if player then
        local meta = player:get_meta();
        meta:set_string('subtitles:display', name);
    end;
end;

function subtitles.Agent:handle_footsteps()
    if not self:get_enabled() then
        return;
    end;

    local player = self:get_player();
    if not player then
        return;
    end;

    local display = self:get_display();
    local player_pos = player:get_pos();
    local objects = minetest.get_objects_inside_radius(player_pos, subtitles.FOOTSTEP_HEAR_DISTANCE);
    for __, object in ipairs(objects) do
        if subtitles.util.object_is_walking(object) and object:get_properties().makes_footstep_sound then
            local feet_pos = subtitles.util.get_feet_pos(object);
            local node = minetest.get_node(feet_pos:offset(0, -0.25, 0));
            local spec = subtitles.util.get_node_sound(node.name, 'footstep');
            if spec then
                local params =
                {
                    to_player         = player;
                    max_hear_distance = subtitles.FOOTSTEP_HEAR_DISTANCE;
                    duration          = subtitles.FOOTSTEP_DURATION;
                    pos               = feet_pos;
                };
                local sound = subtitles.Sound(spec, params, nil);
                display:handle_sound_play(sound);
            end;
        end;
    end;
end;

function subtitles.Agent:step(dtime)
    if self.display then
        self.display:step(dtime);
    end;
end;

function subtitles.Agent:on_leave()
    if self.display then
        self.display:destroy();
    end;
end;
